/*
 * Asterisk -- An open source telephony toolkit.
 *
 * Copyright (C) 2004 - 2005, Pichler Wolfgang.
 *
 * Pichler Wolfgang <wp@commoveo.com>
 *
 * See http://www.asterisk.org for more information about
 * the Asterisk project. Please do not directly contact
 * any of the maintainers of this project for assistance;
 * the project provides a web site, mailing lists and IRC
 * channels for your use.
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License Version 2. See the LICENSE file
 * at the top of the source tree.
 */

/*
 *
 * Application to dump variables into a file
 * 
 */

#include <stdlib.h>
#include <unistd.h>
#include <string.h>

#include "asterisk.h"

ASTERISK_FILE_VERSION(__FILE__, "$Revision: 0.1 $")

#include "asterisk/file.h"
#include "asterisk/logger.h"
#include "asterisk/channel.h"
#include "asterisk/pbx.h"
#include "asterisk/module.h"
#include "asterisk/options.h"
#include "asterisk/utils.h"
#include "asterisk/lock.h"
#include "asterisk/utils.h"
#include "asterisk/app.h"

static char *tdesc = "Dump Variables into a specified file";
static char *app = "Vars2File";
static char *synopsis = "Dump Variables into a specified file";
static char *desc = 
"   Vars2File(filename,var1,var2,...)\n"
"Create the specified file (will overwrite an existing file)\n"
"and will write the into the file the given var names and values.\n"
"The vars and values are separated by an =\n";

STANDARD_LOCAL_USER;

LOCAL_USER_DECL;

static int vars2file_exec(struct ast_channel *chan, void *data)
{
	int argc;
	char *argv[100];
	struct localuser *u;
	char tmp[1024];
	int res;
	FILE *stream;
	char *varvalue;
	int i;
	char workspace[1024];
	
	LOCAL_USER_ADD(u);

	if (data && !ast_strlen_zero(data)) {
		ast_copy_string(tmp, data, sizeof(tmp));
		argc = ast_separate_app_args(tmp, '|', argv, 100);
		if (argc < 2) {
			ast_log(LOG_WARNING,"You have to specifiy by minimum a filename and a variable name");
			res = -1;
		} else {
			if((stream = fopen (argv[0], "w+")) != (FILE *)0) {
				// Now get each variable value and write it to the file
				for (i=1;i<argc;i++) {
					memset(workspace, 0, sizeof(workspace));
					pbx_retrieve_variable(chan, argv[i], &varvalue, workspace, sizeof(workspace), NULL);
					if (varvalue) {
						sprintf(tmp,"%s=%s\n", argv[i], varvalue);
						fputs(tmp, stream);
					} else {
						ast_log(LOG_WARNING, "Unable to get variable called %s.\n",argv[i]);
					}
				}
				fclose(stream);
				res = 0;
			} else {
				ast_log(LOG_WARNING, "Cannot create file %s.\n",argv[0]);
				res = -1;
			}
		}
		
	} else {
		ast_log(LOG_WARNING,"You have to specifiy by minimum a filename and a variable name");
		res = -1;
	}


	LOCAL_USER_REMOVE(u);
	return res;
}

int unload_module(void)
{
	STANDARD_HANGUP_LOCALUSERS;
	return ast_unregister_application(app);
}

int load_module(void)
{
	return ast_register_application(app, vars2file_exec, synopsis, desc);
}

char *description(void)
{
	return tdesc;
}

int usecount(void)
{
	int res;
	STANDARD_USECOUNT(res);
	return res;
}

char *key()
{
	return ASTERISK_GPL_KEY;
}

